--[[
                    ----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----


                       #         ########  #########         #   #    #  #       #  #       #     ####     #   #   #
                       #         #             #            ### ###   #  ##      #  ##      #   ##    ##   #   #   #
                       #         #             #           #  # #  #  #  # #     #  # #     #  ##      ##  #   #   #
                       #         #             #           #   #   #  #  #  #    #  #  #    #  #        #  #   #   #
                       #         ########      #           #   #   #  #  #   #   #  #   #   #  #        #  #   #   #
                       #         #             #           #   #   #  #  #    #  #  #    #  #  #        #  #   #   #
                       #         #             #           #   #   #  #  #     # #  #     # #  ##      ##  #   #   #
                       #         #             #           #   #   #  #  #      ##  #      ##   ##    ##    ### ###
                       ########  ########      #           #   #   #  #  #       #  #       #     ####       #   #


                                                             --==  VANILLA  ==--
                                                       --==  THE BURNING CRUSADE  ==--
                                                       --==  WARLORDS OF DRAENOR  ==--
                                                             --==  LEGION  ==--


                    ----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----(||)----o----


                                                    ----o----(||)----oo----(||)----o----

                                                            v2.00 19th April 2018
                                                    Copyright (C) Taraezor / Chris Birch

                                                    ----o----(||)----oo----(||)----o----


]]

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Localisations and Local Data
--		============================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local authorFlag, authorLocale = true;
local addonName, addonTitle = "LetMinnow";

local find = string.find
local gmatch = string.gmatch or string.gfind		-- LUA 5.0 -> 5.1 renamed gfind to gmatch. This in case 5.0 is used
local gsub = string.gsub
local lower = string.lower
local random = random
local sub = string.sub
local tonumber = tonumber
local tostring = tostring

local CreateMacro = CreateMacro
local EditMacro = EditMacro
local EquipItemByName = EquipItemByName				-- TBC
local GetAddOnMetadata = GetAddOnMetadata
local GetItemCount = GetItemCount					-- TBC
local GetItemInfo = GetItemInfo
local GetMacroIndexByName = GetMacroIndexByName
local SendChatMessage = SendChatMessage

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Build / Version Setup
--		=====================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local buildVersion = GetBuildInfo()
local vanilla, tbc, booleanTrue = false, false;
local _, _, buildVersionMajor, buildVersionMinor, _ = find( buildVersion, "(%d+)\.(%d+)" )
local _, _, luaVersionMinor = find( (_VERSION or "Lua 5.0"), "%d+\.(%d+)" )
if ( buildVersionMajor == "1" ) then
	vanilla = true
	booleanTrue = 1
elseif ( buildVersionMajor == "2" ) then
	tbc = true
	booleanTrue = 1
else
	booleanTrue = true
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Colour Printing - Blue Theme
--		===============
--
--------------------------------------------------------------------------------------------------------------------------------------------

local pc_colour_Prefix		= "\124cFF1E90FF"
local pc_colour_Highlight	= "\124cFF4169E1"
local pc_colour_PlainText	= "\124cFF87CEFA"

local function printPC( message )
	if message then
		DEFAULT_CHAT_FRAME:AddMessage( pc_colour_Prefix.. ( addonTitle or addonName ).. ": ".. pc_colour_PlainText.. message.. "\124r" )
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Language Localisation
--		=====================
--
--------------------------------------------------------------------------------------------------------------------------------------------

local locale = authorLocale or GetLocale()

local L = {}
setmetatable( L, { __index = function( L, key) return key end } )

if locale == "deDE" then
	L["Fishing"] = "Angeln"
	L["Commands"] = "Befehle"
	L["Turn on / off"] = "An / Ausschalten"
	L["Toggle"] = "Schalter"
	L["Yell about your Fishing Pole"] = "Schreie über deine Angelrute"
	L["Show the version number"] = "Zeigt die Versionsnummer an"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "Benötigt The Burning Crusade (2.4.3) oder höher"
	L["On"] = "Ein"
	L["Off"] = "Aus"
	L["Yell"] = "Schrei"
	L["Silent"] = "Leise"
	L["Version: "] = "Version: "

elseif locale == "esES" then
	L["Fishing"] = "Pesca"
	L["Commands"] = "Comandos"
	L["Turn on / off"] = "Encender / apagar"
	L["Toggle"] = "Cambiar"
	L["Yell about your Fishing Pole"] = "Grita sobre tu caña de pescar"
	L["Show the version number"] = "Mostrar el número de versión"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "Requiere The Burning Crusade (2.4.3) o superior"
	L["On"] = "Encendido"
	L["Off"] = "Apagado"
	L["Yell"] = "Grito"
	L["Silent"] = "Silencio"
	L["Version: "] = "Versión: "

elseif locale == "frFR" then
	L["Fishing"] = "Pêche"
	L["Commands"] = "Commandes"
	L["Turn on / off"] = "Allume / éteint"
	L["Toggle"] = "Interrupteur"
	L["Yell about your Fishing Pole"] = "Criez sur votre canne à pêche"
	L["Show the version number"] = "Afficher le numéro de version"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "Nécessite The Burning Crusade (2.4.3) ou supérieur"
	L["On"] = "Allumé"
	L["Off"] = "Éteint"
	L["Yell"] = "Hurler"
	L["Silent"] = "Silencieux"
	L["Version: "] = "Version: "

elseif locale == "itIT" then
	L["Fishing"] = "Pesca"
	L["Commands"] = "Comandi"
	L["Turn on / off"] = "Accendi / spegni"
	L["Toggle"] = "Interruttore"
	L["Yell about your Fishing Pole"] = "Urla della tua canna da pesca"
	L["Show the version number"] = "Mostra il numero di versione"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "Richiede The Burning Crusade (2.4.3) o superiore"
	L["On"] = "Acceso"
	L["Off"] = "Spento"
	L["Yell"] = "Urlo"
	L["Silent"] = "Silenzioso"
	L["Version: "] = "Versione: "
	
elseif locale == "koKR" then
	L["Fishing"] = "낚시"
	L["Commands"] = "명령들"
	L["Turn on / off"] = "켜기 / 끄기"
	L["Toggle"] = "스위치"
	L["Yell about your Fishing Pole"] = "낚시대에 대해 소리 지르십시오."
	L["Show the version number"] = "버전 번호 표시"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "The Burning Crusade (2.4.3) 이상 필요"
	L["On"] = "켜다"
	L["Off"] = "끄다"
	L["Yell"] = "외침"
	L["Silent"] = "조용한"
	L["Version: "] = "번역: "

elseif locale == "ptBR" then
	L["Fishing"] = "Pesca"
	L["Commands"] = "Comandos"
	L["Turn on / off"] = "Ligar / desligar"
	L["Toggle"] = "Interruptor"
	L["Yell about your Fishing Pole"] = "Grite sobre sua vara de pescar"
	L["Show the version number"] = "Mostrar o número da versão"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "Requer The Burning Crusade (2.4.3) ou superior"
	L["On"] = "Ligado"
	L["Off"] = "Desligado"
	L["Yell"] = "Grito"
	L["Silent"] = "Silencioso"
	L["Version: "] = "Versão: "

elseif locale == "ruRU" then
	L["Fishing"] = "Рыбная ловля"
	L["Commands"] = "Команды"
	L["Turn on / off"] = "Включение / выключение"
	L["Toggle"] = "Переключатель"
	L["Yell about your Fishing Pole"] = "Улыбайтесь о своем рыболовном столбе"
	L["Show the version number"] = "Вывести номер версии"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "Требуется The Burning Crusade (2.4.3) или выше"
	L["On"] = "Включен"
	L["Off"] = "Выключен"
	L["Yell"] = "Вопить"
	L["Silent"] = "бесшумный"
	L["Version: "] = "Версия: "
	
elseif locale == "zhCN" then
	L["Fishing"] = "钓鱼"
	L["Commands"] = "命令"
	L["Turn on / off"] = "打开/关闭"
	L["Toggle"] = "开关"
	L["Yell about your Fishing Pole"] = "大喊你的钓竿"
	L["Show the version number"] = "显示版本号"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "需要 The Burning Crusade (2.4.3) 或更高"
	L["On"] = "启用"
	L["Off"] = "禁用"
	L["Yell"] = "叫喊"
	L["Silent"] = "无声"
	L["Version: "] = "版： "
	
elseif locale == "zhTW" then
	L["Fishing"] = "釣魚"
	L["Commands"] = "命令"
	L["Turn on / off"] = "打開/關閉"
	L["Toggle"] = "開關"
	L["Yell about your Fishing Pole"] = "大喊你的釣竿"
	L["Show the version number"] = "顯示版本號"
	L["Requires The Burning Crusade (2.4.3) or higher"] = "需要 The Burning Crusade (2.4.3) 或更高"
	L["On"] = "啟用"
	L["Off"] = "禁用"
	L["Yell"] = "叫喊"
	L["Silent"] = "無聲"
	L["Version: "] = "版： "
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Fishing Poles
--		=============
--
--------------------------------------------------------------------------------------------------------------------------------------------

local lm_FishingPole = {
			"6256",		-- Fishing Pole
			"6365",		-- Strong Fishing Pole
			"6366",		-- Darkwood Fishing Pole
			"12225",	-- Blump Family Fishing Pole
			"46337",	-- Staat's Fishing Pole
			"120163",	-- Thruk's Fishing Pole
			"52678",	-- Jonathan's Fishing Pole
			"6367",		-- Big Iron Fishing Pole
			"19022",	-- Nat Pagle's Extreme Angler FC-5000
			"19970",	-- Arcanite Fishing Pole
			"25978",	-- Seth's Graphite Fishing Pole
			"45858",	-- Nat's Lucky Fishing Pole
			"43651",	-- Crafty's Pole
			"45992",	-- Jeweled Fishing Pole
			"118381",	-- Ephemeral Fishing Pole
			"44050",	-- Mastercraft Kalu'ak Fishing Pole
			"45991",	-- Bone Fishing Pole
			"84660",	-- Pandaren Fishing Pole
			"84661",	-- Dragon Fishing Pole
			"116825",	-- Savage Fishing Pole
			"116826",	-- Draenic Fishing Pole
			"133755",	-- Underlight Angler
}

local numberOfFishingPoles = 22		-- Avoid using #lm_FishingPole as AddOn won't load in Vanilla

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Global Function
--		===============
--
--------------------------------------------------------------------------------------------------------------------------------------------

local macroName = L["Fishing"]
local theMacro = "#Showtooltip\n/run LetMinnow()\n/cast Fishing"

function LetMinnow()	-- Never get to here in Vanilla

	if ( LetMinnowDB.on == false ) then return end

	local index, tries = 0, 0

	while true do
		index, tries = random( 1, numberOfFishingPoles ), tries + 1
		if ( GetItemCount( tonumber( lm_FishingPole[ index ] ) ) > 0 ) then

			EquipItemByName( tonumber( lm_FishingPole[ index ] ) )

			if ( tbc == false ) then
			
				-- In TBC we could not select an actual fishing pole item icon. We reply upon the #Showtooltip cast/spell icon
				
				if ( index <= 7 ) then
					EditMacro( macroName, macroName, "INV_Fishingpole_02", theMacro )
				elseif ( index <= 13 ) then
					EditMacro( macroName, macroName, "INV_Fishingpole_01", theMacro )
				elseif ( index <= 15 ) then
					EditMacro( macroName, macroName, "INV_Fishingpole_05", theMacro )
				elseif ( index == 16 ) then
					EditMacro( macroName, macroName, "INV_Fishingpole_03", theMacro )
				elseif ( index == 17 ) then
					EditMacro( macroName, macroName, "Achievement_Profession_Fishing_JourneymanFisher", theMacro )
				elseif ( index == 18 ) then
					EditMacro( macroName, macroName, "INV_Misc_2H_bamboo_fishing", theMacro )
				elseif ( index == 19 ) then
					EditMacro( macroName, macroName, "INV_Misc_2H_panda_fishing", theMacro )
				elseif ( index <= 21 ) then
					EditMacro( macroName, macroName, "INV_Misc_2H_DraenorFishingPole_B_01", theMacro )
				else
					EditMacro( macroName, macroName, "Achievement_Profession_Fishing_NorthrendAngler", theMacro )
				end
			end

			if ( LetMinnowDB.yell == "y" ) then
				local name = GetItemInfo( tonumber( lm_FishingPole[ index ] ) )
				if name then
					SendChatMessage( name, "YELL" )
				end
			end

			return
		end

		if ( tries > 150 ) then
			break
		end
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Events
--		======
--
--------------------------------------------------------------------------------------------------------------------------------------------

local function OnEventHandler( frame, theEvent, arg )

	if ( vanilla == true ) then return end
	
	-- Prior to Patch 4.0.1, the globals "event", "arg1", etc were automagically setup by Blizzard. No variables were passed
	-- From Patch 4.0.1 "self", "event", and varag "args" are passed. Renamed here from normal conventions to avoid conflicts

	if ( theEvent == "VARIABLES_LOADED" ) or ( event == "VARIABLES_LOADED" ) then

		if not LetMinnowDB then LetMinnowDB = {} end
		local yell = LetMinnowDB.yell or "y"
		local onOff = LetMinnowDB.on or "y"
		LetMinnowDB = {}
		LetMinnowDB.on = onOff
		LetMinnowDB.yell = yell

		addonTitle = GetAddOnMetadata( addonName, "Title" )

 	elseif ( theEvent == "UPDATE_MACROS" ) or ( event == "UPDATE_MACROS" ) then
	
		if ( GetMacroIndexByName( macroName ) == 0 ) then
			if ( tbc == true ) then
				CreateMacro( macroName, 1, theMacro )	-- In TBC we must make do with the #Showtooltip cast/spell icon
			else
				CreateMacro( macroName, "Trade_Fishing", theMacro )
			end
		end
	end
end

local eventFrame = CreateFrame( "Frame" )
eventFrame:RegisterEvent( "UPDATE_MACROS" )
eventFrame:RegisterEvent( "VARIABLES_LOADED" )
eventFrame:SetScript( "OnEvent", OnEventHandler )

--------------------------------------------------------------------------------------------------------------------------------------------
--
--		Slash Commands
--		==============
--
--------------------------------------------------------------------------------------------------------------------------------------------

local soundHandle;

SLASH_LetMinnow1, SLASH_LetMinnow2, SLASH_LetMinnow3 = "/letminnow", "/minnow", "/lm"

SlashCmdList[ "LetMinnow" ] = function( options )

	local options1, secondParm;
	for v in gmatch( options, "(%S+)" ) do
		v = lower(v)
		if not options1 then
			options1 = sub( v,1,1 )
		else
			secondParm = sub( v,1,2 )
			break
		end
	end

	if ( vanilla == true ) and ( options1 ~= "v") then options1 = nil end

	if not options1 or ( options1 == "?" ) then
		printPC( pc_colour_Highlight.. L["Commands"].. ":\n"..
			pc_colour_Highlight.. "o".. pc_colour_PlainText.. " = ".. L["Turn on / off"].. ". (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "y".. pc_colour_PlainText.. " = ".. L["Yell about your Fishing Pole"].. ". (".. L["Toggle"].. ")\n"..
			pc_colour_Highlight.. "v".. pc_colour_PlainText.. " = ".. L["Show the version number"].. "\n")
		if ( vanilla == true ) then
			printPC( pc_colour_Highlight.. L["Requires The Burning Crusade (2.4.3) or higher"] )
		end

	elseif ( options1 == "o" ) then
		if not secondParm then
			LetMinnowDB.on = ( LetMinnowDB.on == "y" ) and "n" or "y"
		end
		printPC( pc_colour_Highlight.. "o".. pc_colour_PlainText.. " = ".. pc_colour_Highlight.. 
					( ( LetMinnowDB.on == "y" ) and L["On"] or L["Off"] ) )

	elseif ( options1 == "y" ) then
		if not secondParm then
			LetMinnowDB.yell = ( LetMinnowDB.yell == "y" ) and "n" or "y"
		end
		printPC( pc_colour_Highlight.. "y".. pc_colour_PlainText.. " = ".. pc_colour_Highlight.. 
					( ( LetMinnowDB.yell == "y" ) and L["Yell"] or L["Silent"] ) )

	elseif ( options1 == "v" ) then
		local version = GetAddOnMetadata( addonName, "Version" )
		printPC( pc_colour_Highlight.. L["Version: "].. pc_colour_PlainText.. " = ".. pc_colour_Highlight.. version )

	-- Easter Eggs. Enjoy!

	elseif ( options1 == "z" ) then
		if ( buildVersionMajor >= "5" ) then
			if soundHandle then 
				StopSound( soundHandle, 5000 )
				soundHandle = nil
				return
			end
			if ( random( 2 ) == 2 ) then
				soundID = 12816					-- Grizzly Hills
			else
				soundID = 14907					-- WotLK/Northrend Dalaran
			end
			if ( buildVersionMajor > "7" ) or ( ( buildVersionMajor == "7" ) and ( buildVersionMinor >= "3" ) ) then
				_, soundHandle = PlaySound( soundID )
			else
				_, soundHandle = PlaySoundKitID( soundID )
			end
		else
			PlaySoundFile( "Sound\\Music\\GlueScreenMusic\\BCCredits_Lament_of_the_Highborne.mp3" )
		end
	end
end